local Telestone = Class(function(self, inst)
self.inst = inst

self.onMark = nil
self.onRecall = nil
self.onReset = nil
self.level = nil
self.caninteractfn = nil

self.level = nil
self.marked = nil
self.location = nil 
self.locations = {}

end)

function Telestone:OnSave()
	
	local data = {locations = {}}
	
	for i = 1, #self.locations do
		data.locations[i] = self.locations[i]
	end
	
	return data
	
end

function Telestone:OnLoad(data)
	
	for i = 1, #data.locations do
		self.locations[i] = data.locations[i]
	end
	
	self.level = SaveGameIndex:GetSaveGameName(SaveGameIndex:GetCurrentMode(), SaveGameIndex:GetCurrentSaveSlot())
	
	for i = 1, #self.locations do
		if self.level == self.locations[i] then
			self.index = i
		end
	end
	
	self.location = Point(0,0,0)
	
	if self.index then
		self.location.x = self.locations[self.index+1]
		self.location.y = self.locations[self.index+2]
		self.location.z = self.locations[self.index+3]
	
		self.marked = true
	end
	
end

function Telestone:CollectInventoryActions(doer, actions)
  if self:CanInteract() then
       if (self:IsMarked() and TheInput:IsKeyDown(KEY_CTRL)) then
	        table.insert(actions, ACTIONS.RESET)
       elseif self:IsMarked() then
	        table.insert(actions, ACTIONS.RECALL)		
	   else
		    table.insert(actions, ACTIONS.MARK)
	   end		
   end
end

function Telestone:Mark(inst, doer)

	if self.onMark then
		self.onMark(self.inst, doer)
	end
	
	   self.level = SaveGameIndex:GetSaveGameName(SaveGameIndex:GetCurrentMode(), SaveGameIndex:GetCurrentSaveSlot())
	   self.location = Point(GetPlayer().Transform:GetWorldPosition())
	   self.mark = SpawnPrefab("mark")
	   self.mark.Transform:SetPosition(self.location.x, self.location.y, self.location.z)
	   self.marked = true
	   
	   table.insert(self.locations, self.level)
	   table.insert(self.locations, self.location.x)
	   table.insert(self.locations, self.location.y)
	   table.insert(self.locations, self.location.z)
	   
end

function Telestone:Reset(inst, doer)

	if self.onReset then
		self.onReset(self.inst, doer)
	end
    
	for i = 1, #self.locations do
		if self.level == self.locations[i] then
			self.index = i
		end
	end
	
	table.remove(self.locations, self.index)
	table.remove(self.locations, self.index)
	table.remove(self.locations, self.index)
	table.remove(self.locations, self.index)
	
	self.marked = false
	
	GetPlayer():PushEvent("reset_stone", {doer = doer})
	
end

function Telestone:CanInteract()
	if self.caninteractfn then
		return self.caninteractfn(self.inst)
	else
		return true
	end
end

function Telestone:Recall(doer)
	if self.onRecall then 
		self.onRecall(self.inst, doer)
	end
	
	self:Teleport(doer)
	
	if doer.components.leader then
		for follower,v in pairs(doer.components.leader.followers) do
			self:Teleport(follower)
		end
	end

	if doer.components.inventory then
		for k,item in pairs(doer.components.inventory.itemslots) do
			if item.components.leader then
				for follower,v in pairs(item.components.leader.followers) do
					self:Teleport(follower)
				end
			end
		end
		
		for k,equipped in pairs(doer.components.inventory.equipslots) do
			if equipped and equipped.components.container then
				local container = equipped.components.container
				for j,item in pairs(container.slots) do
					if item.components.leader then
						for follower,v in pairs(item.components.leader.followers) do
							self:Teleport(follower)
						end
					end
				end
			end
		end
	end
	
	for i = 1, #self.locations do
		if self.level == self.locations[i] then
			self.index = i
		end		
	end
	
	table.remove(self.locations, self.index)
	table.remove(self.locations, self.index)
	table.remove(self.locations, self.index)
	table.remove(self.locations, self.index)
	
	self.marked = false

	GetPlayer():PushEvent("recall_stone", {doer = doer})
	
end

function Telestone:Teleport(obj)

    local offset = 2.0
    local angle = math.random()*360
 
		self.location.x = self.location.x + math.sin(angle)*offset
		self.location.z = self.location.z + math.cos(angle)*offset
		
		if obj.Physics then
			obj.Physics:Teleport(self.location.x, self.location.y, self.location.z)
		elseif obj.Transform then
			obj.Transform:SetPosition(self.location.x, self.location.y, self.location.z)	
		end	
		
end

function Telestone:IsMarked()

	return self.marked
	
end

return Telestone