local assets =
{
	Asset("ANIM", "anim/rabbit_hole.zip"),
}

local prefabs = 
{
	"rabbit",
	"smallmeat",
}


local function dig_up(inst, chopper)
	if inst.components.spawner:IsOccupied() then
		inst.components.lootdropper:SpawnLootPrefab("rabbit")
	end
	inst:Remove()
end

local function startspawning(inst)
    if inst.components.spawner and (GetSeasonManager() and not GetSeasonManager():IsSpring()) then
        inst.components.spawner:SpawnWithDelay(60 + math.random(120) )
    end
end

local function stopspawning(inst)
    if inst.components.spawner then
        inst.components.spawner:CancelSpawning()
    end
end

local function onoccupied(inst)
    if GetClock():IsDay() then
        startspawning(inst)
    end
    if inst.spring then
    	inst.AnimState:PlayAnimation("idle_flooded")
    	inst.wet_prefix = STRINGS.WET_PREFIX.RABBITHOLE
    	inst.always_wet = true
    end
end

local function GetChild(inst)
	return "rabbit"
end

local function SetSpringMode(inst)
	stopspawning(inst)
	inst.springtask = nil
	inst.spring = true
	if inst.components.spawner:IsOccupied() then
		inst.AnimState:PlayAnimation("idle_flooded")
		inst.wet_prefix = STRINGS.WET_PREFIX.RABBITHOLE
		inst.always_wet = true
	end
end

local function SetNormalMode(inst)
	inst.AnimState:PlayAnimation("idle")
	if GetClock():IsDay() and inst.components.spawner and not inst.components.spawner:IsSpawnPending() then
        startspawning(inst)
    end
    inst.normaltask = nil
    inst.spring = false
   	inst.wet_prefix = STRINGS.WET_PREFIX.NONE
   	inst.always_wet = false
end

  
local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()

    anim:SetBank("rabbithole")
    anim:SetBuild("rabbit_hole")
    anim:PlayAnimation("idle")
	--anim:SetOrientation( ANIM_ORIENTATION.OnGround )
	anim:SetLayer( LAYER_BACKGROUND )
	anim:SetSortOrder( 3 )

	local minimap = inst.entity:AddMiniMapEntity()
	minimap:SetIcon( "rabbit_hole.tex" )

	inst:AddComponent( "spawner" )
	inst.components.spawner:Configure( "rabbit", TUNING.RABBIT_RESPAWN_TIME)
	inst.components.spawner.childfn = GetChild

	inst:AddTag("cattoy")
	
	inst.components.spawner:SetOnOccupiedFn(onoccupied)
	inst.components.spawner:SetOnVacateFn(stopspawning)
    
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.DIG)
    inst.components.workable:SetOnFinishCallback(dig_up)
    inst.components.workable:SetWorkLeft(1)
    
	inst:ListenForEvent( "dusktime", function() stopspawning(inst) end, GetWorld())
	inst:ListenForEvent( "daytime", function() startspawning(inst) end, GetWorld())

	inst.spring = GetSeasonManager():IsSpring()
	if inst.spring then
		inst:DoTaskInTime(.1, function(inst) SetSpringMode(inst) end)
	end
	inst:ListenForEvent( "rainstart", function(it, data)
		if GetSeasonManager() and GetSeasonManager():IsSpring() then
			inst.springtask = inst:DoTaskInTime(math.random(TUNING.MIN_RABBIT_HOLE_TRANSITION_TIME, TUNING.MAX_RABBIT_HOLE_TRANSITION_TIME), SetSpringMode)
		end
	end, GetWorld())

	inst:ListenForEvent("seasonChange", function(it, data)
		if data.season ~= SEASONS.SPRING then
			inst.springtask = inst:DoTaskInTime(math.random(TUNING.MIN_RABBIT_HOLE_TRANSITION_TIME, TUNING.MAX_RABBIT_HOLE_TRANSITION_TIME), SetSpringMode)
		end
	end, GetWorld())

	inst.OnSave = function(inst, data)
        data.makespring = (inst.springtask ~= nil) and true or inst.spring
        data.makenormal = (inst.normaltask ~= nil) and true or inst.spring
    end        
    
    inst.OnLoad = function(inst, data)
        if data then
			if data.makespring then
				SetSpringMode(inst)
			end
			if data.makenormal then
				SetNormalMode(inst)
			end
	    end 
    end
	
    inst:AddComponent("inspectable")
    inst.components.inspectable.getstatus =  function(inst)
    	if inst.spring then
    		return "SPRING"
    	end
   	end
	
    return inst
end

return Prefab( "common/objects/rabbithole", fn, assets, prefabs ) 
